/**
 *
 *  EGLPORT.H
 *  Copyright (C) 2011-2013 Scott R. Smith
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  in the Software without restriction, including without limitation the rights
 *  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  copies of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 *  THE SOFTWARE.
 *
 */

#ifndef EGLPORT_H
#define EGLPORT_H

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef __EGL__
extern GLfloat col1[4*4];
inline void assign4x4(GLfloat *q, GLfloat a11, GLfloat a12, GLfloat a13, GLfloat a14, GLfloat a21, GLfloat a22, GLfloat a23, GLfloat a24, GLfloat a31, GLfloat a32, GLfloat a33, GLfloat a34, GLfloat a41, GLfloat a42, GLfloat a43, GLfloat a44)
{ q[0]=a11; q[1]=a12; q[2]=a13; q[3]=a14; q[4]=a21; q[5]=a22; q[6]=a23; q[7]=a24; q[8]=a31; q[9]=a32; q[10]=a33; q[11]=a34; q[12]=a41; q[13]=a42; q[14]=a43; q[15]=a44;};
/*inline void assign3x4(GLfloat *q, GLfloat a11, GLfloat a12, GLfloat a13, GLfloat a21, GLfloat a22, GLfloat a23, GLfloat a31, GLfloat a32, GLfloat a33, GLfloat a41, GLfloat a42, GLfloat a43)
{ q[0]=a11; q[1]=a12; q[2]=a13; q[3]=a21; q[4]=a22; q[5]=a23; q[6]=a31; q[7]=a32; q[8]=a33; q[9]=a41; q[10]=a42; q[11]=a43;};*/
#define assign3x4(q, a11, a12, a13, a21, a22, a23, a31, a32, a33, a41, a42, a43) \
{ q[0]=a11; q[1]=a12; q[2]=a13; q[3]=a21; q[4]=a22; q[5]=a23; q[6]=a31; q[7]=a32; q[8]=a33; q[9]=a41; q[10]=a42; q[11]=a43; }

inline void assign2x4(GLfloat *q, GLfloat a11, GLfloat a12, GLfloat a21, GLfloat a22, GLfloat a31, GLfloat a32, GLfloat a41, GLfloat a42)
{ q[0]=a11; q[1]=a12; q[2]=a21; q[3]=a22; q[4]=a31; q[5]=a32; q[6]=a41; q[7]=a42; };
//#define Color3f(r, g, b) Color4f(r, g, b, 1)
//#define Color3fv(v) Color4f(v[0], v[1], v[2], 1)
//#define Color4fv(v) Color4f(v[0], v[1], v[2], v[3])
#define glColor4fv(a) glColor4f(a[0], a[1], a[2], a[3])
#define glColor3fv(a) glColor4f(a[0], a[1], a[2], 1.0f)
#define glColor3f(a,b,c) glColor4f(a, b, c, 1.0f)
#define GL_CLAMP     GL_CLAMP_TO_EDGE
#define glOrtho      glOrthof
void gluPerspective(GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar);
inline void Color4f(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
{
 col1[0+0] = col1[4+0] = col1[8+0] = col1[12+0] = r;
 col1[0+1] = col1[4+1] = col1[8+1] = col1[12+1] = g;
 col1[0+2] = col1[4+2] = col1[8+2] = col1[12+2] = b;
 col1[0+3] = col1[4+3] = col1[8+3] = col1[12+3] = a;
 glColor4f(r, g, b, a);
}

#endif



/** Defines (in every case choose only one) */
/**     Common: */
/**         DEBUG : enable additional error monitoring per EGL function call */
/**     Native display and window system for use with EGL */
/**         USE_EGL_SDL : used for access to a SDL X11 window */
/**     Platform: settings that are specific to that device */
/**         PANDORA (USE_GLES1 or USE_GLES2) */
/**         WIZ     (USE_GLES1) */
/**         CAANOO  (USE_GLES1) */
/**         RPI     (USE_GLES1 or USE_GLES2) */
/**     GLES Version */
/**         USE_GLES1 : EGL for use with OpenGL-ES 1.X contexts */
/**         USE_GLES2 : EGL for use with OpenGL-ES 2.0 contexts */

/** Public API */
void    EGL_Close                   ( void );
int8_t  EGL_Open                    ( uint16_t width, uint16_t height );
void    EGL_SwapBuffers             ( void );

/** Simple Examples  */
/**     Raw mode:
            EGL_Open( window_width, window_height );
            do while(!quit) {
                ... run app
                EGL_SwapBuffers();
            }
            EGL_Close();
*/
/**     X11/SDL mode:
            SDL_Init( SDL_INIT_VIDEO );
            SDL_Surface* screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE|SDL_FULLSCREEN);
            EGL_Open( window_width, window_height );
            do while(!quit) {
                ... run app
                EGL_SwapBuffers();
            }
            EGL_Close();
            SDL_Quit();
*/

#if defined(DEBUG)
#define GET_EGLERROR(FUNCTION)               \
    FUNCTION;                                \
    {                                        \
        CheckEGLErrors(__FILE__, __LINE__);  \
    }
#else
#define GET_EGLERROR(FUNCTION) FUNCTION;
#endif

#define peglQueryString(A,B)                    GET_EGLERROR(eglQueryString(A,B))
#define peglDestroyContext(A,B)                 GET_EGLERROR(eglDestroyContext(A,B))
#define peglDestroySurface(A,B)                 GET_EGLERROR(eglDestroySurface(A,B))
#define peglTerminate(A)                        GET_EGLERROR(eglTerminate(A))
#define peglSwapBuffers(A,B)                    GET_EGLERROR(eglSwapBuffers(A,B))
#define peglGetDisplay(A)                       GET_EGLERROR(eglGetDisplay(A))
#define peglBindAPI(A)                          GET_EGLERROR(eglBindAPI(A))
#define peglCreateContext(A,B,C,D)              GET_EGLERROR(eglCreateContext(A,B,C,D))
#define peglCreateWindowSurface(A,B,C,D)        GET_EGLERROR(eglCreateWindowSurface(A,B,C,D))
#define peglInitialize(A,B,C)                   GET_EGLERROR(eglInitialize(A,B,C))
#define peglMakeCurrent(A,B,C,D)                GET_EGLERROR(eglMakeCurrent(A,B,C,D))
#define peglChooseConfig(A,B,C,D,E)             GET_EGLERROR(eglChooseConfig(A,B,C,D,E))
#define peglSwapInterval(A,B)                   GET_EGLERROR(eglSwapInterval(A,B))

#ifdef __cplusplus
}
#endif

#endif /* EGLPORT_H */
